//
// Author:           Paul Watt
// Created:          1/15/2009
//
// Setup program for the CxxTest App Wizard for VC++ (2005|2008|2010|2012|2013|2015)
//
//

// This is the sub-category used to store the wizards.
var categoryDir = "CodeOfTheDamned";

main();

function main()
{
  // Detect the proper version and location of Visual Studio:
  
  // Try to install for each version.
  // If there are no valid versions, display an error and exit.
  
  // If you have both licensed versions and express versions of 
  // Visual Studio, you may need to edit the arrays below to only
  // register the licensed versions. 
  // It does not gracefully test for which type of install you have.

  var vsCount = 5;
  var vsVerList = new Array("8.0","9.0","10.0","11.0","12.0","14.0");
  var vsNameList = new Array("VS2005","VS2008","VS2010","VS2012","VS2013","VS2015");
  var vsPropExt = new Array("vsprops", "vsprops", "props", "props", "props", "props");
  var vsVer;
  
  // Decode command line arguments
  var bDebug = false;
  var bElevated = false;
  var Args = WScript.Arguments;
  for(var i = 0; i < Args.length; i++)
  {
    if(Args(i) == "/debug")
      bDebug = true;
    else if(Args(i) == "/elevated")
      bElevated = true;
  }

  // See if UAC is enabled
  var Shell = WScript.CreateObject("Shell.Application");
  if(!bElevated && Shell.IsRestricted("System", "EnableLUA"))
  {
    // Check that the script is being run interactively.
    if(!WScript.Interactive)
    {
      WScript.Echo("ERROR: Elevation required.");
      return;
    }

    // Now relaunch the script, using the "RunAs" verb to elevate
    var strParams = "\"" + WScript.ScriptFullName + "\"";
    if (bDebug)
      strParams += " /debug";
    strParams += " /elevated";
    Shell.ShellExecute(WScript.FullName, strParams, null, "RunAs");
    return;
  }

  // Create shell object
  var WSShell = WScript.CreateObject("WScript.Shell");
  // Create file system object
  var FileSys = WScript.CreateObject("Scripting.FileSystemObject");

  // Keep track of the versions of Visual Studio the wizards have been installed for.
  var installedVersions;

  var verIndex;
  for (verIndex = 0; verIndex < vsCount; verIndex++)
  {
    var strProductDir;
    try
    {
      var strVCKey = "HKLM\\Software\\Microsoft\\VisualStudio\\" + vsVerList[verIndex] + "\\Setup\\VC\\ProductDir";
      strProductDir = WSShell.RegRead(strVCKey);
      vsVer = vsVerList[verIndex];

      if (null == installedVersions)
        installedVersions = vsNameList[verIndex];
      else
        installedVersions += ", " + vsNameList[verIndex];
    }
    catch(e)
    {
      try
      {
        var strVCKey_x64 = "HKLM\\Software\\Wow6432Node\\Microsoft\\VisualStudio\\" + vsVerList[verIndex] + "\\Setup\\VC\\ProductDir";
        strProductDir = WSShell.RegRead(strVCKey_x64);
        vsVer = vsVerList[verIndex];

        if (null == installedVersions)
          installedVersions = vsNameList[verIndex];
        else
          installedVersions += ", " + vsNameList[verIndex];
      }
      catch(e)
      {
        // Skip this version, look for the next version in the list.
        // An error will be reported at the end if Visual Studio has not been installed.
        continue;                         
      }
    }

    // Get the folder containing the script file
    var strValue = FileSys.GetParentFolderName(WScript.ScriptFullName);
    if(strValue == null || strValue == "")
      strValue = ".";

    var strSourceFolderList = new Array("AppWiz", "CodeWiz");
    var strWizNameList = new Array("CxxTestWiz", "CxxTestFixture");
    var strWizDirList  = new Array("vcprojects", "VCAddClass");
    var index;
    for (index = 0; index < 2; index++)
    {
      // Create the source dir name.
      var strSourceFolder = FileSys.BuildPath(strValue, "Files\\" + strSourceFolderList[index]);
      if(bDebug)
        WScript.Echo("Source: " + strSourceFolder);

      if(!FileSys.FolderExists(strSourceFolder))
      {
        WScript.Echo("ERROR: Cannot find Wizard folder (should be: " + strSourceFolder + ")");
        return;
      }

      // Create the destination dir name.
      var strDestFolder = FileSys.BuildPath(strProductDir, strWizDirList[index]);
      if(bDebug)
        WScript.Echo("Destination: " + strDestFolder);
      if(!FileSys.FolderExists(strDestFolder))
      {
        WScript.Echo("ERROR: Cannot find destination folder (should be: " + strDestFolder + ")");
        return;
      }


      // Copy files
      var WizName = strWizNameList[index];
      try
      {
        var strSrc = FileSys.BuildPath(strSourceFolder, WizName + ".ico");
        var strDest = FileSys.BuildPath(strDestFolder, WizName + ".ico");
        FileSys.CopyFile(strSrc, strDest);

        strSrc = FileSys.BuildPath(strSourceFolder, WizName + ".vsdir");
        strDest = FileSys.BuildPath(strDestFolder, WizName + ".vsdir");
        FileSys.CopyFile(strSrc, strDest);
      }
      catch(e)
      {
        var strError = "no info";
        if(e.description.length != 0)
          strError = e.description;
        WScript.Echo("ERROR: Cannot copy file (" + strSrc + "): " + strError);
        return;
      }

      // Read and write CxxTestWiz.vsz, add engine version and replace path when found
      try
      {
        var strSrc = FileSys.BuildPath(strSourceFolder, WizName + ".vsz");
        var strDest = FileSys.BuildPath(strDestFolder, WizName + ".vsz");

        var ForReading = 1;
        var fileSrc = FileSys.OpenTextFile(strSrc, ForReading);
        if(fileSrc == null)
        {
          WScript.Echo("ERROR: Cannot open source file " + strSrc);
          return;
        }

        var ForWriting = 2;
        var fileDest = FileSys.OpenTextFile(strDest, ForWriting, true);
        if(fileDest == null)
        {
          WScript.Echo("ERROR: Cannot open destination file" + strDest);
          return;
        }

        while(!fileSrc.AtEndOfStream)
        {
          var strLine = fileSrc.ReadLine();
          if(strLine.indexOf("Wizard=VsWizard.VsWizardEngine") != -1)
            strLine = "Wizard=VsWizard.VsWizardEngine." + vsVer;
          else if(strLine.indexOf("WIZARD_VERSION") != -1)
            strLine = "Param=\"WIZARD_VERSION = " + vsVer + "\"";
          fileDest.WriteLine(strLine);
        }

        fileSrc.Close();
        fileDest.Close();
      }
      catch(e)
      {
        var strError = "no info";
        if(e.description.length != 0)
          strError = e.description;
        WScript.Echo("ERROR: Cannot read and write " + WizName + ".vsz (" + strError + ")");
        return;
      }

      // Create CxxTest Wizard folder
      var strDestWizFolder = "";
      try
      {
      	strDestWizFolder = FileSys.BuildPath(strDestFolder, categoryDir);
        if(!FileSys.FolderExists(strDestWizFolder))
          FileSys.CreateFolder(strDestWizFolder);
        if(bDebug)
          WScript.Echo("Cxx Folder: " + strDestWizFolder);
      }
      catch(e)
      {
        var strError = "no info";
        if(e.description.length != 0)
          strError = e.description;
        WScript.Echo("ERROR: Cannot create Cxx Wizard folder (" + strError + ")");
        return;
      }

      // Read and write additional CxxTestWiz.vsdir, add path to the wizard location
      try
      {
        var strSrc = FileSys.BuildPath(strSourceFolder, WizName + ".vsdir");
        var strDest = FileSys.BuildPath(strDestWizFolder, WizName + ".vsdir");

        var ForReading = 1;
        var fileSrc = FileSys.OpenTextFile(strSrc, ForReading);
        if(fileSrc == null)
        {
          WScript.Echo("ERROR: Cannot open source file " + strSrc);
          return;
        }

        var ForWriting = 2;
        var fileDest = FileSys.OpenTextFile(strDest, ForWriting, true);
        if(fileDest == null)
        {
          WScript.Echo("ERROR: Cannot open destination file" + strDest);
          return;
        }

        var searchStr = WizName + ".vsz|";
        while(!fileSrc.AtEndOfStream)
        {
          var strLine = fileSrc.ReadLine();
          if(strLine.indexOf(searchStr) != -1)
          {
            strLine = "..\\" + strLine;
          }

          fileDest.WriteLine(strLine);
        }

        fileSrc.Close();
        fileDest.Close();
      }
      catch(e)
      {
        var strError = "no info";
        if(e.description.length != 0)
          strError = e.description;
        WScript.Echo("ERROR: Cannot read and write " + categoryDir + "\\" + WizName + ".vsdir (" + strError + ")");
        return;
      }

      // Copy the test wizard files into the wizard folder for Visual Studio
      var strVcWizFolder = FileSys.BuildPath(strProductDir, "vcwizards");
      var strDestFolder = FileSys.BuildPath(strProductDir, "vcwizards\\" + strSourceFolderList[index] + "\\" + categoryDir + "\\" + WizName);
      if(bDebug)
        WScript.Echo("Destination: " + strDestFolder);

      // Make sure the visual studio wizard folder exists.
      if(!FileSys.FolderExists(strVcWizFolder))
      {
        WScript.Echo("ERROR: Cannot find destination folder (should be: " + strDestFolder + ")");
        return;
      }
      else
      {
        // Verify the sub-directory structure exists as well.
        var strWizFolder = FileSys.BuildPath(strVcWizFolder, strSourceFolderList[index]);
        if(!FileSys.FolderExists(strWizFolder))
        {
          FileSys.CreateFolder(strWizFolder);
        }

        var strCatFolder = FileSys.BuildPath(strWizFolder, categoryDir);
        if(!FileSys.FolderExists(strCatFolder))
        {
          FileSys.CreateFolder(strCatFolder);
        }

        if(!FileSys.FolderExists(strDestFolder))
        {
          FileSys.CreateFolder(strDestFolder);
        }
      }

      // Make sure the destination folder exists.
      if(!FileSys.FolderExists(strDestFolder))
      {
        WScript.Echo("ERROR: Cannot find destination folder (should be: " + strDestFolder + ")");
        return;
      }

      // Copy folders, the new folder will be created by the script call below.
      try
      {
        var strCopySrcCmd  = FileSys.BuildPath(strSourceFolder, "\\*");
        var strCopyDestCmd = FileSys.BuildPath(strDestFolder, "\\");

        FileSys.CopyFolder(strCopySrcCmd, strCopyDestCmd, true);

   	    // Copy the correct property files for the current version of visual studio.
        if (index == 0)
        {
          var strSrc = FileSys.BuildPath(strSourceFolder, "CxxTest." + vsPropExt[verIndex]);
          var strDest = FileSys.BuildPath(strDestFolder, "CxxTest." + vsPropExt[verIndex]);
          FileSys.CopyFile(strSrc, strDest);
        }
      }
      catch(e)
      {
        var strError = "no info";
        if(e.description.length != 0)
          strError = e.description;
        WScript.Echo("ERROR: Cannot copy folders (" + strError + ")");
        return;
      }
    }
  }

  if (installedVersions == null)
  {
    WScript.Echo("ERROR: Cannot find where Visual Studio is installed.");
  }
  else
  {
    WScript.Echo("CxxTest Wizards successfully installed for: " + installedVersions + "!");
  }
}
